@@ -1,11 +1,11 @@
 import logging
+import warnings
 
 from django.conf import settings
 from django.contrib import messages
 from django.contrib.auth import logout
 from django.contrib.auth.decorators import login_required
 from django.core.exceptions import ObjectDoesNotExist
-from django.core.urlresolvers import get_callable
 from django.core.urlresolvers import reverse_lazy
 from django.utils.decorators import method_decorator
 from django.utils.translation import ugettext_lazy as _
@@ -17,6 +17,7 @@ from mama_cas.models import ProxyTicket
 from mama_cas.models import ProxyGrantingTicket
 from mama_cas.exceptions import InvalidTicket
 from mama_cas.exceptions import ValidationError
+from mama_cas.utils import get_callable
 
 
 logger = logging.getLogger(__name__)
@@ -60,19 +61,13 @@ class ValidateTicketMixin(object):
     """
     View mixin providing ticket validation methods.
     """
-    def validate_service_ticket(self, request):
+    def validate_service_ticket(self, service, ticket, pgturl, renew):
         """
         Validate a service ticket string. Return a triplet containing
         a ``ServiceTicket`` and an optional ``ProxyGrantingTicket``,
         or a ``ValidationError`` subclass if ticket validation failed.
         """
-        service = request.GET.get('service')
-        ticket = request.GET.get('ticket')
-        renew = bool(request.GET.get('renew'))
-        pgturl = request.GET.get('pgtUrl')
-
         logger.debug("Service validation request received for %s" % ticket)
-
         # Check for proxy tickets passed to /serviceValidate
         if ticket and ticket.startswith(ProxyTicket.TICKET_PREFIX):
             e = InvalidTicket('Proxy tickets cannot be validated'
@@ -97,17 +92,13 @@ class ValidateTicketMixin(object):
                 pgt = None
             return st, pgt, None
 
-    def validate_proxy_ticket(self, request):
+    def validate_proxy_ticket(self, service, ticket, pgturl):
         """
         Validate a proxy ticket string. Return a 4-tuple containing a
         ``ProxyTicket``, an optional ``ProxyGrantingTicket`` and a list
         of proxies through which authentication proceeded, or a
         ``ValidationError`` subclass if ticket validation failed.
         """
-        service = request.GET.get('service')
-        ticket = request.GET.get('ticket')
-        pgturl = request.GET.get('pgtUrl')
-
         logger.debug("Proxy validation request received for %s" % ticket)
         try:
             pt = ProxyTicket.objects.validate_ticket(ticket, service)
@@ -133,15 +124,12 @@ class ValidateTicketMixin(object):
                 pgt = None
             return pt, pgt, proxies, None
 
-    def validate_proxy_granting_ticket(self, request):
+    def validate_proxy_granting_ticket(self, pgt, target_service):
         """
         Validate a proxy granting ticket string. Return an ordered
         pair containing a ``ProxyTicket``, or a ``ValidationError``
         subclass if ticket validation failed.
         """
-        pgt = request.GET.get('pgt')
-        target_service = request.GET.get('targetService')
-
         logger.debug("Proxy ticket request received for %s using %s" %
                      (target_service, pgt))
         try:
@@ -163,9 +151,9 @@ class CustomAttributesMixin(object):
     """
     def get_custom_attributes(self, ticket):
         """
-        Build a list of user attributes from the ``User`` object, the
-        user profile object or a custom callback callable. Attributes
-        are specified with one or more settings:
+        Build a dictionary of user attributes from the ``User``
+        object, the user profile object or custom callbacks using one
+        or more of these settings:
 
         ``MAMA_CAS_USER_ATTRIBUTES``
             A dict of name and ``User`` attribute values. The name can
@@ -177,10 +165,10 @@ class CustomAttributesMixin(object):
             can be any meaningful string, while the attribute must
             correspond with an attribute on the user profile object.
 
-        ``MAMA_CAS_ATTRIBUTES_CALLBACK``
-            A string representation of a custom callable that returns
-            a dict containing attributes. The callable is provided a
-            single argument of the ``User``.
+        ``MAMA_CAS_ATTRIBUTE_CALLBACKS``
+            A tuple of dotted paths to callables that return a dict of
+            name and attribute values. Each callable is provided a
+            single argument of the authenticated ``User``.
 
         All attributes are returned as a single dictionary.
         """
@@ -190,6 +178,12 @@ class CustomAttributesMixin(object):
         attributes = {}
 
         user_attr_list = getattr(settings, 'MAMA_CAS_USER_ATTRIBUTES', {})
+        if user_attr_list:
+            warnings.warn(
+                "The MAMA_CAS_USER_ATTRIBUTES setting is deprecated and "
+                "will be removed in MamaCAS 1.0. Use the "
+                "MAMA_CAS_ATTRIBUTE_CALLBACKS setting instead.",
+                PendingDeprecationWarning)
         for name, attr in user_attr_list.items():
             try:
                 attributes[name] = getattr(user, attr)
@@ -203,15 +197,21 @@ class CustomAttributesMixin(object):
         else:
             profile_attr_list = getattr(settings,
                                         'MAMA_CAS_PROFILE_ATTRIBUTES', {})
+            if profile_attr_list:
+                warnings.warn(
+                    "The MAMA_CAS_PROFILE_ATTRIBUTES setting is deprecated "
+                    "and will be removed in MamaCAS 1.0. Use the "
+                    "MAMA_CAS_ATTRIBUTE_CALLBACKS setting instead.",
+                    PendingDeprecationWarning)
             for name, attr in profile_attr_list.items():
                 try:
                     attributes[name] = getattr(profile, attr)
                 except AttributeError:
                     logger.error("Profile has no attribute named '%s'" % attr)
 
-        callback_str = getattr(settings, 'MAMA_CAS_ATTRIBUTES_CALLBACK', None)
-        if callback_str is not None:
-            callback = get_callable(callback_str)
+        callbacks = getattr(settings, 'MAMA_CAS_ATTRIBUTE_CALLBACKS', ())
+        for path in callbacks:
+            callback = get_callable(path)
             attributes.update(callback(user))
 
         return attributes
@@ -232,6 +232,7 @@ class LogoutUserMixin(object):
 
         3. Call logout() to end the session and purge all session data.
         """
+        logger.debug("Logout request received for %s" % request.user)
         if request.user.is_authenticated():
             ServiceTicket.objects.consume_tickets(request.user)
             ProxyTicket.objects.consume_tickets(request.user)
