@@ -17,18 +17,24 @@ from django.utils.translation import ugettext_lazy as _
 
 import requests
 
+from mama_cas.compat import gevent
 from mama_cas.compat import user_model
-from mama_cas.exceptions import InvalidRequest
-from mama_cas.exceptions import InvalidTicket
-from mama_cas.exceptions import InvalidService
-from mama_cas.exceptions import InternalError
 from mama_cas.exceptions import BadPgt
+from mama_cas.exceptions import InvalidProxyCallback
+from mama_cas.exceptions import InvalidRequest
+from mama_cas.exceptions import InvalidService
+from mama_cas.exceptions import InvalidTicket
 from mama_cas.request import SingleSignOutRequest
 from mama_cas.utils import add_query_params
 from mama_cas.utils import is_scheme_https
 from mama_cas.utils import clean_service_url
 from mama_cas.utils import is_valid_service_url
 
+if gevent:
+    from gevent.pool import Pool
+    from gevent import monkey
+    monkey.patch_all(thread=False, select=False)
+
 
 logger = logging.getLogger(__name__)
 
@@ -115,11 +121,9 @@ class TicketManager(models.Manager):
 
         A custom management command is provided that executes this method
         on all applicable models by running ``manage.py cleanupcas``.
-        This command should be run on a regular basis to prevent invalid
-        tickets from causing storage or performance issues.
         """
         for ticket in self.filter(Q(consumed__isnull=False) |
-                                  Q(expires__lte=now())):
+                                  Q(expires__lte=now())).order_by('-expires'):
             try:
                 ticket.delete()
             except models.ProtectedError:
@@ -195,9 +199,28 @@ class ServiceTicketManager(TicketManager):
         Send a single sign-out request to each service accessed by a
         specified user. This is called at logout when single sign-out
         is enabled.
+
+        If gevent is installed, asynchronous requests will be sent.
+        Otherwise, synchronous requests will be sent. By default there
+        is no limit to the number of concurrent jobs. Setting
+        ``MAMA_CAS_ASYNC_CONCURRENCY`` limits concurrent requests to
+        the specified value.
         """
-        for ticket in self.filter(user=user, consumed__gte=user.last_login):
-            ticket.request_sign_out()
+        def spawn(ticket, pool=None):
+            if pool is not None:
+                return pool.spawn(ticket.request_sign_out)
+            return gevent.spawn(ticket.request_sign_out)
+
+        tickets = list(self.filter(user=user, consumed__gte=user.last_login))
+
+        if gevent:
+            size = getattr(settings, 'MAMA_CAS_ASYNC_CONCURRENCY', None)
+            pool = Pool(size) if size else None
+            requests = [spawn(t, pool=pool) for t in tickets]
+            gevent.joinall(requests)
+        else:
+            for ticket in tickets:
+                ticket.request_sign_out()
 
 
 class ServiceTicket(Ticket):
@@ -258,7 +281,6 @@ class ProxyTicket(Ticket):
 
     service = models.CharField(_('service'), max_length=255)
     granted_by_pgt = models.ForeignKey('ProxyGrantingTicket',
-                                       on_delete=models.PROTECT,
                                        verbose_name=_('granted by proxy-granting ticket'))
 
     class Meta:
@@ -284,7 +306,7 @@ class ProxyGrantingTicketManager(TicketManager):
         try:
             if validate:
                 self.validate_callback(pgturl, pgtid, pgtiou)
-        except InternalError as e:
+        except InvalidProxyCallback as e:
             # pgtUrl validation failed, so nothing has been created
             logger.warning("%s %s" % (e.code, e))
             return None
@@ -309,7 +331,7 @@ class ProxyGrantingTicketManager(TicketManager):
         """
         # Ensure the scheme is HTTPS before proceeding
         if not is_scheme_https(url):
-            raise InternalError("Proxy callback %s is not HTTPS" % url)
+            raise InvalidProxyCallback("Proxy callback %s is not HTTPS" % url)
 
         # Connect to proxy callback URL, checking the SSL certificate
         url_params = add_query_params(url, {'pgtId': pgtid, 'pgtIou': pgtiou})
@@ -318,19 +340,20 @@ class ProxyGrantingTicketManager(TicketManager):
             r = requests.get(url_params, verify=verify, timeout=3.0)
         except requests.exceptions.SSLError:
             msg = "SSL cert validation failed for proxy callback %s" % url
-            raise InternalError(msg)
+            raise InvalidProxyCallback(msg)
         except requests.exceptions.ConnectionError:
             msg = "Error connecting to proxy callback %s" % url
-            raise InternalError(msg)
+            raise InvalidProxyCallback(msg)
         except requests.exceptions.Timeout:
             msg = "Timeout connecting to proxy callback %s" % url
-            raise InternalError(msg)
+            raise InvalidProxyCallback(msg)
 
         # Check the returned HTTP status code
         try:
             r.raise_for_status()
         except requests.exceptions.HTTPError as e:
-            raise InternalError("Proxy callback %s returned %s" % (url, e))
+            msg = "Proxy callback %s returned %s" % (url, e)
+            raise InvalidProxyCallback(msg)
 
     def validate_ticket(self, ticket, service):
         """
