@@ -3,18 +3,13 @@ from django.contrib.auth.models import User
 
 
 class ExceptionBackend(ModelBackend):
-    """
-    This authentication backend raises an exception on authentication
-    for testing purposes.
-    """
+    """Raise an exception on authentication for testing purposes."""
     def authenticate(self, username=None, password=None):
         raise Exception
 
 
 class CaseInsensitiveBackend(ModelBackend):
-    """
-    A case-insenstitive authentication backend.
-    """
+    """A case-insenstitive authentication backend."""
     def authenticate(self, username=None, password=None):
         try:
             user = User.objects.get(username__iexact=username)
