@@ -6,6 +6,8 @@ from mama_cas.utils import is_scheme_https
 from mama_cas.utils import clean_service_url
 from mama_cas.utils import is_valid_service_url
 from mama_cas.utils import redirect
+from mama_cas.utils import to_bool
+from mama_cas.utils import get_callable
 
 
 class UtilsTests(TestCase):
@@ -88,3 +90,23 @@ class UtilsTests(TestCase):
         """
         r = redirect('http')
         self.assertEqual('/login', r['Location'])
+
+    def test_to_bool(self):
+        """
+        Any string value should evaluate as ``True``. Empty strings
+        or strings of whitespace should evaluate as ``False``.
+        """
+        self.assertTrue(to_bool('true'))
+        self.assertTrue(to_bool('1'))
+        self.assertFalse(to_bool(None))
+        self.assertFalse(to_bool(''))
+        self.assertFalse(to_bool('   '))
+
+    def test_get_callable_invalid_path(self):
+        """An invalid callable path should raise an ``ImportError``."""
+        self.assertRaises(ImportError, get_callable, 'invalid_import_path')
+
+    def test_get_callable_invalid_import(self):
+        """An invalid callable should raise an ``ImportError``."""
+        self.assertRaises(ImportError, get_callable,
+                          'mama_cas.tests.callback.invalid_callback')
