@@ -8,6 +8,7 @@ except ImportError:  # pragma: no cover
     from urlparse import parse_qsl, urlparse, urlunparse
 
 from django.conf import settings
+from django.core.exceptions import PermissionDenied
 from django.core import urlresolvers
 from django.http import HttpResponseRedirect
 
@@ -77,9 +78,38 @@ def redirect(to, *args, **kwargs):
     except urlresolvers.NoReverseMatch:
         if '/' not in to and '.' not in to:
             to = urlresolvers.reverse('cas_login')
+        elif not is_valid_service_url(to):
+            raise PermissionDenied()
 
     if params:
         to = add_query_params(to, params)
 
     logger.debug("Redirecting to %s" % to)
     return HttpResponseRedirect(to)
+
+
+def to_bool(str):
+    """
+    Converts a given string to a boolean value. Leading and trailing
+    whitespace is ignored, so strings of whitespace are evaluated as
+    ``False``.
+    """
+    if str:
+        return bool(str.strip())
+    return False
+
+
+def get_callable(path):
+    """Returns a callable from a given dotted path."""
+    try:
+        module_path, callable_name = path.rsplit('.', 1)
+    except ValueError:
+        raise ImportError("%s doesn't look like a callable path" % path)
+
+    module = __import__(module_path, fromlist=[''])
+
+    try:
+        return getattr(module, callable_name)
+    except AttributeError:
+        raise ImportError("Could not import %s from module %s" %
+                          (callable_name, module_path))
