@@ -9,6 +9,8 @@ from django.views.generic import FormView
 from django.views.generic import TemplateView
 from django.views.generic import View
 
+import defusedxml.ElementTree as etree
+
 from mama_cas.compat import get_username
 from mama_cas.forms import LoginForm
 from mama_cas.mixins import CasResponseMixin
@@ -21,10 +23,12 @@ from mama_cas.models import ProxyTicket
 from mama_cas.models import ServiceTicket
 from mama_cas.response import ValidationResponse
 from mama_cas.response import ProxyResponse
+from mama_cas.response import SamlValidationResponse
 from mama_cas.utils import add_query_params
 from mama_cas.utils import clean_service_url
 from mama_cas.utils import is_valid_service_url
 from mama_cas.utils import redirect
+from mama_cas.utils import to_bool
 
 
 logger = logging.getLogger(__name__)
@@ -56,13 +60,11 @@ class LoginView(NeverCacheMixin, LogoutUserMixin, FormView):
            the specified service.
         """
         service = request.GET.get('service')
-        renew = bool(request.GET.get('renew'))
-        gateway = bool(request.GET.get('gateway'))
+        renew = to_bool(request.GET.get('renew'))
+        gateway = to_bool(request.GET.get('gateway'))
 
         if renew:
             logger.debug("Renew request received by credential requestor")
-            self.logout_user(request)
-            return redirect('cas_login', params={'service': service})
         elif gateway and service:
             logger.debug("Gateway request received by credential requestor")
             if request.user.is_authenticated():
@@ -154,6 +156,10 @@ class WarnView(NeverCacheMixin, LoginRequiredMixin, TemplateView):
     def get(self, request, *args, **kwargs):
         service = request.GET.get('service')
         ticket = request.GET.get('ticket')
+
+        if not service or not is_valid_service_url(service):
+            return redirect('cas_login')
+
         msg = _("Do you want to access %s as %s?") % (clean_service_url(service),
                                                       request.user)
         messages.info(request, msg)
@@ -168,21 +174,28 @@ class LogoutView(NeverCacheMixin, LogoutUserMixin, View):
     """
     (2.3) End a client's single sign-on session.
 
-    When this view is accessed, an existing single sign-on session is
-    ended, requiring a new single sign-on session to be established
-    for future authentication attempts.
+    Accessing this view ends an existing single sign-on session,
+    requiring a new single sign-on session to be established for
+    future authentication attempts.
 
-    If ``url`` is specified, by default it will be displayed to the
-    user as a recommended link to follow. This behavior can be altered
-    by setting ``MAMA_CAS_FOLLOW_LOGOUT_URL`` to ``True``, which will
-    redirect the client to the specified URL.
+    [CAS 3.0] If ``service`` is specified and
+    ``MAMA_CAS_FOLLOW_LOGOUT_URL`` is ``True``, the client will be
+    redirected to the specified service URL.
+
+    [CAS 1.0, CAS 2.0] If ``url`` is specified, by default it will be
+    displayed to the user as a recommended link to follow. This
+    behavior can be altered by setting ``MAMA_CAS_FOLLOW_LOGOUT_URL``
+    to ``True``, which redirects the client to the specified URL.
     """
     def get(self, request, *args, **kwargs):
-        logger.debug("Logout request received for %s" % request.user)
-        self.logout_user(request)
+        service = request.GET.get('service')
         url = request.GET.get('url')
-        if url and is_valid_service_url(url):
-            if getattr(settings, 'MAMA_CAS_FOLLOW_LOGOUT_URL', False):
+        follow_url = getattr(settings, 'MAMA_CAS_FOLLOW_LOGOUT_URL', True)
+        self.logout_user(request)
+        if service and follow_url:
+            return redirect(service)
+        elif url and is_valid_service_url(url):
+            if follow_url:
                 return redirect(url)
             msg = _("The application provided this link to follow: %s") % url
             messages.success(request, msg)
@@ -204,7 +217,12 @@ class ValidateView(NeverCacheMixin, ValidateTicketMixin, View):
     primary credentials, not from an existing single sign-on session.
     """
     def get(self, request, *args, **kwargs):
-        st, pgt, error = self.validate_service_ticket(request)
+        service = request.GET.get('service')
+        ticket = request.GET.get('ticket')
+        renew = to_bool(request.GET.get('renew'))
+
+        st, pgt, error = self.validate_service_ticket(service, ticket,
+                                                      None, renew)
         if st:
             content = "yes\n%s\n" % get_username(st.user)
         else:
@@ -234,7 +252,13 @@ class ServiceValidateView(NeverCacheMixin, ValidateTicketMixin,
     response_class = ValidationResponse
 
     def get_context_data(self, **kwargs):
-        st, pgt, error = self.validate_service_ticket(self.request)
+        service = self.request.GET.get('service')
+        ticket = self.request.GET.get('ticket')
+        pgturl = self.request.GET.get('pgtUrl')
+        renew = to_bool(self.request.GET.get('renew'))
+
+        st, pgt, error = self.validate_service_ticket(service, ticket,
+                                                      pgturl, renew)
         attributes = self.get_custom_attributes(st)
         return {'ticket': st, 'pgt': pgt, 'error': error,
                 'attributes': attributes}
@@ -263,13 +287,19 @@ class ProxyValidateView(NeverCacheMixin, ValidateTicketMixin,
     response_class = ValidationResponse
 
     def get_context_data(self, **kwargs):
+        service = self.request.GET.get('service')
         ticket = self.request.GET.get('ticket')
+        pgturl = self.request.GET.get('pgtUrl')
+        renew = to_bool(self.request.GET.get('renew'))
+
         if not ticket or ticket.startswith(ProxyTicket.TICKET_PREFIX):
             # If no ticket parameter is present, attempt to validate it
             # anyway so the appropriate error is raised
-            t, pgt, proxies, error = self.validate_proxy_ticket(self.request)
+            t, pgt, proxies, error = self.validate_proxy_ticket(service,
+                                                                ticket, pgturl)
         else:
-            t, pgt, error = self.validate_service_ticket(self.request)
+            t, pgt, error = self.validate_service_ticket(service, ticket,
+                                                         pgturl, renew)
             proxies = None
         attributes = self.get_custom_attributes(t)
         return {'ticket': t, 'pgt': pgt, 'proxies': proxies,
@@ -290,5 +320,39 @@ class ProxyView(NeverCacheMixin, ValidateTicketMixin, CasResponseMixin, View):
     response_class = ProxyResponse
 
     def get_context_data(self, **kwargs):
-        pt, error = self.validate_proxy_granting_ticket(self.request)
+        pgt = self.request.GET.get('pgt')
+        target_service = self.request.GET.get('targetService')
+
+        pt, error = self.validate_proxy_granting_ticket(pgt, target_service)
         return {'ticket': pt, 'error': error}
+
+
+class SamlValidateView(NeverCacheMixin, ValidateTicketMixin,
+                       CustomAttributesMixin, View):
+    """
+    (4.2) Check the validity of a service ticket provided by a
+    SAML 1.1 request document provided by a HTTP POST. [CAS 3.0]
+    """
+    response_class = SamlValidationResponse
+    content_type = 'text/xml'
+
+    def post(self, request, *args, **kwargs):
+        context = self.get_context_data(**kwargs)
+        return self.render_to_response(context)
+
+    def render_to_response(self, context):
+        return self.response_class(context, content_type=self.content_type)
+
+    def get_context_data(self, **kwargs):
+        target = self.request.GET.get('target')
+
+        try:
+            root = etree.parse(self.request, forbid_dtd=True).getroot()
+            ticket = root.find('.//{urn:oasis:names:tc:SAML:1.0:protocol}AssertionArtifact').text
+        except (etree.ParseError, ValueError, AttributeError):
+            ticket = None
+
+        st, pgt, error = self.validate_service_ticket(target, ticket, None, None)
+        attributes = self.get_custom_attributes(st)
+        return {'ticket': st, 'pgt': pgt, 'error': error,
+                'attributes': attributes}
